@@ -4,14 +4,20 @@ import logging
 import os
 import time
 import traceback
-from typing import Any
+from collections.abc import Iterable
+from typing import TYPE_CHECKING, Any, ClassVar
 
 import numpy as np
 
-from ..conventions import cf_encoder
-from ..core import indexing
-from ..core.pycompat import is_duck_dask_array
-from ..core.utils import FrozenDict, NdimSizeLenMixin, is_remote_uri
+from xarray.conventions import cf_encoder
+from xarray.core import indexing
+from xarray.core.pycompat import is_duck_dask_array
+from xarray.core.utils import FrozenDict, NdimSizeLenMixin, is_remote_uri
+
+if TYPE_CHECKING:
+    from io import BufferedIOBase
+
+    from xarray.core.dataset import Dataset
 
 # Create a logger object, but don't add any handlers. Leave that to user code.
 logger = logging.getLogger(__name__)
@@ -80,9 +86,9 @@ def robust_getitem(array, key, catch=Exception, max_retries=6, initial_delay=500
 class BackendArray(NdimSizeLenMixin, indexing.ExplicitlyIndexed):
     __slots__ = ()
 
-    def __array__(self, dtype=None):
+    def get_duck_array(self, dtype: np.typing.DTypeLike = None):
         key = indexing.BasicIndexer((slice(None),) * self.ndim)
-        return np.asarray(self[key], dtype=dtype)
+        return self[key]  # type: ignore [index]
 
 
 class AbstractDataStore:
@@ -369,24 +375,50 @@ class BackendEntrypoint:
     - ``guess_can_open`` method: it shall return ``True`` if the backend is able to open
       ``filename_or_obj``, ``False`` otherwise. The implementation of this
       method is not mandatory.
+
+    Attributes
+    ----------
+
+    open_dataset_parameters : tuple, default: None
+        A list of ``open_dataset`` method parameters.
+        The setting of this attribute is not mandatory.
+    description : str, default: ""
+        A short string describing the engine.
+        The setting of this attribute is not mandatory.
+    url : str, default: ""
+        A string with the URL to the backend's documentation.
+        The setting of this attribute is not mandatory.
     """
 
-    open_dataset_parameters: tuple | None = None
-    """list of ``open_dataset`` method parameters"""
+    open_dataset_parameters: ClassVar[tuple | None] = None
+    description: ClassVar[str] = ""
+    url: ClassVar[str] = ""
+
+    def __repr__(self) -> str:
+        txt = f"<{type(self).__name__}>"
+        if self.description:
+            txt += f"\n  {self.description}"
+        if self.url:
+            txt += f"\n  Learn more at {self.url}"
+        return txt
 
     def open_dataset(
         self,
-        filename_or_obj: str | os.PathLike,
-        drop_variables: tuple[str] | None = None,
+        filename_or_obj: str | os.PathLike[Any] | BufferedIOBase | AbstractDataStore,
+        *,
+        drop_variables: str | Iterable[str] | None = None,
         **kwargs: Any,
-    ):
+    ) -> Dataset:
         """
         Backend open_dataset method used by Xarray in :py:func:`~xarray.open_dataset`.
         """
 
         raise NotImplementedError
 
-    def guess_can_open(self, filename_or_obj: str | os.PathLike):
+    def guess_can_open(
+        self,
+        filename_or_obj: str | os.PathLike[Any] | BufferedIOBase | AbstractDataStore,
+    ) -> bool:
         """
         Backend open_dataset method used by Xarray in :py:func:`~xarray.open_dataset`.
         """
@@ -394,4 +426,5 @@ class BackendEntrypoint:
         return False
 
 
-BACKEND_ENTRYPOINTS: dict[str, type[BackendEntrypoint]] = {}
+# mapping of engine name to (module name, BackendEntrypoint Class)
+BACKEND_ENTRYPOINTS: dict[str, tuple[str | None, type[BackendEntrypoint]]] = {}
