@@ -1,17 +1,19 @@
 from __future__ import annotations
 
 import warnings
-from typing import TYPE_CHECKING, Any, Callable, Hashable, Iterable, Sequence
+from collections.abc import Hashable, Iterable, Sequence
+from typing import TYPE_CHECKING, Any, Callable
 
-import numpy as np
-
-from ._reductions import DataArrayResampleReductions, DatasetResampleReductions
-from .groupby import DataArrayGroupByBase, DatasetGroupByBase, GroupBy
-from .types import InterpOptions, T_Xarray
+from xarray.core._aggregations import (
+    DataArrayResampleAggregations,
+    DatasetResampleAggregations,
+)
+from xarray.core.groupby import DataArrayGroupByBase, DatasetGroupByBase, GroupBy
+from xarray.core.types import Dims, InterpOptions, T_Xarray
 
 if TYPE_CHECKING:
-    from .dataarray import DataArray
-    from .dataset import Dataset
+    from xarray.core.dataarray import DataArray
+    from xarray.core.dataset import Dataset
 
 RESAMPLE_DIM = "__resample_dim__"
 
@@ -37,40 +39,22 @@ class Resample(GroupBy[T_Xarray]):
         resample_dim: Hashable | None = None,
         **kwargs,
     ) -> None:
-
         if dim == resample_dim:
             raise ValueError(
                 f"Proxy resampling dimension ('{resample_dim}') "
                 f"cannot have the same name as actual dimension ('{dim}')!"
             )
         self._dim = dim
-        self._resample_dim = resample_dim
 
         super().__init__(*args, **kwargs)
 
-    def _flox_reduce(self, dim, keep_attrs: bool | None = None, **kwargs) -> T_Xarray:
-
-        from .dataarray import DataArray
-
-        kwargs.setdefault("method", "cohorts")
-
-        # now create a label DataArray since resample doesn't do that somehow
-        repeats = []
-        for slicer in self._group_indices:
-            assert isinstance(slicer, slice)
-            stop = (
-                slicer.stop
-                if slicer.stop is not None
-                else self._obj.sizes[self._group_dim]
-            )
-            repeats.append(stop - slicer.start)
-        labels = np.repeat(self._unique_coord.data, repeats)
-        group = DataArray(labels, dims=(self._group_dim,), name=self._unique_coord.name)
-
-        result = super()._flox_reduce(
-            dim=dim, group=group, keep_attrs=keep_attrs, **kwargs
-        )
-        result = self._maybe_restore_empty_groups(result)
+    def _flox_reduce(
+        self,
+        dim: Dims,
+        keep_attrs: bool | None = None,
+        **kwargs,
+    ) -> T_Xarray:
+        result = super()._flox_reduce(dim=dim, keep_attrs=keep_attrs, **kwargs)
         result = result.rename({RESAMPLE_DIM: self._group_dim})
         return result
 
@@ -195,7 +179,7 @@ class Resample(GroupBy[T_Xarray]):
 
 
 # https://github.com/python/mypy/issues/9031
-class DataArrayResample(Resample["DataArray"], DataArrayGroupByBase, DataArrayResampleReductions):  # type: ignore[misc]
+class DataArrayResample(Resample["DataArray"], DataArrayGroupByBase, DataArrayResampleAggregations):  # type: ignore[misc]
     """DataArrayGroupBy object specialized to time resampling operations over a
     specified dimension
     """
@@ -258,8 +242,8 @@ class DataArrayResample(Resample["DataArray"], DataArrayGroupByBase, DataArrayRe
         if self._dim in combined.coords:
             combined = combined.drop_vars(self._dim)
 
-        if self._resample_dim in combined.dims:
-            combined = combined.rename({self._resample_dim: self._dim})
+        if RESAMPLE_DIM in combined.dims:
+            combined = combined.rename({RESAMPLE_DIM: self._dim})
 
         return combined
 
@@ -287,11 +271,11 @@ class DataArrayResample(Resample["DataArray"], DataArrayGroupByBase, DataArrayRe
         resampled : DataArray
         """
         self._obj = self._drop_coords()
-        return self.mean(self._dim)
+        return self.mean(None if self._dim is None else [self._dim])
 
 
 # https://github.com/python/mypy/issues/9031
-class DatasetResample(Resample["Dataset"], DatasetGroupByBase, DatasetResampleReductions):  # type: ignore[misc]
+class DatasetResample(Resample["Dataset"], DatasetGroupByBase, DatasetResampleAggregations):  # type: ignore[misc]
     """DatasetGroupBy object specialized to resampling a specified dimension"""
 
     def map(
@@ -334,7 +318,16 @@ class DatasetResample(Resample["Dataset"], DatasetGroupByBase, DatasetResampleRe
         applied = (func(ds, *args, **kwargs) for ds in self._iter_grouped())
         combined = self._combine(applied)
 
-        return combined.rename({self._resample_dim: self._dim})
+        # If the aggregation function didn't drop the original resampling
+        # dimension, then we need to do so before we can rename the proxy
+        # dimension we used.
+        if self._dim in combined.coords:
+            combined = combined.drop_vars(self._dim)
+
+        if RESAMPLE_DIM in combined.dims:
+            combined = combined.rename({RESAMPLE_DIM: self._dim})
+
+        return combined
 
     def apply(self, func, args=(), shortcut=None, **kwargs):
         """
@@ -355,10 +348,10 @@ class DatasetResample(Resample["Dataset"], DatasetGroupByBase, DatasetResampleRe
     def reduce(
         self,
         func: Callable[..., Any],
-        dim: None | Hashable | Iterable[Hashable] = None,
+        dim: Dims = None,
         *,
-        axis: None | int | Sequence[int] = None,
-        keep_attrs: bool = None,
+        axis: int | Sequence[int] | None = None,
+        keep_attrs: bool | None = None,
         keepdims: bool = False,
         shortcut: bool = True,
         **kwargs: Any,
@@ -372,7 +365,7 @@ class DatasetResample(Resample["Dataset"], DatasetGroupByBase, DatasetResampleRe
             Function which can be called in the form
             `func(x, axis=axis, **kwargs)` to return the result of collapsing
             an np.ndarray over an integer valued axis.
-        dim : Hashable or Iterable of Hashable, optional
+        dim : "...", str, Iterable of Hashable or None, optional
             Dimension(s) over which to apply `func`.
         keep_attrs : bool, optional
             If True, the datasets's attributes (`attrs`) will be copied from
@@ -406,4 +399,4 @@ class DatasetResample(Resample["Dataset"], DatasetGroupByBase, DatasetResampleRe
         resampled : Dataset
         """
         self._obj = self._drop_coords()
-        return self.mean(self._dim)
+        return self.mean(None if self._dim is None else [self._dim])
