@@ -1,18 +1,13 @@
 from __future__ import annotations
 
 import datetime
+from collections.abc import Hashable, Iterable, Sequence
 from typing import (
     TYPE_CHECKING,
     Any,
     Callable,
-    Hashable,
-    Iterable,
-    List,
     Literal,
-    Protocol,
-    Sequence,
     SupportsIndex,
-    Tuple,
     TypeVar,
     Union,
 )
@@ -22,6 +17,7 @@ import pandas as pd
 from packaging.version import Version
 
 if TYPE_CHECKING:
+    from numpy._typing import _SupportsDType
     from numpy.typing import ArrayLike
 
     from xarray.backends.common import BackendEntrypoint
@@ -54,19 +50,12 @@ if TYPE_CHECKING:
     _ShapeLike = Union[SupportsIndex, Sequence[SupportsIndex]]
     _DTypeLikeNested = Any  # TODO: wait for support for recursive types
 
-    # once NumPy 1.21 is minimum version, use NumPys definition directly
-    # 1.20 uses a non-generic Protocol (like we define here for simplicity)
-    class _SupportsDType(Protocol):
-        @property
-        def dtype(self) -> np.dtype:
-            ...
-
     # Xarray requires a Mapping[Hashable, dtype] in many places which
     # conflics with numpys own DTypeLike (with dtypes for fields).
     # https://numpy.org/devdocs/reference/typing.html#numpy.typing.DTypeLike
     # This is a copy of this DTypeLike that allows only non-Mapping dtypes.
     DTypeLikeSave = Union[
-        np.dtype,
+        np.dtype[Any],
         # default data type (float64)
         None,
         # array-scalar types and generic types
@@ -74,15 +63,15 @@ if TYPE_CHECKING:
         # character codes, type strings or comma-separated fields, e.g., 'float64'
         str,
         # (flexible_dtype, itemsize)
-        Tuple[_DTypeLikeNested, int],
+        tuple[_DTypeLikeNested, int],
         # (fixed_dtype, shape)
-        Tuple[_DTypeLikeNested, _ShapeLike],
+        tuple[_DTypeLikeNested, _ShapeLike],
         # (base_dtype, new_dtype)
-        Tuple[_DTypeLikeNested, _DTypeLikeNested],
+        tuple[_DTypeLikeNested, _DTypeLikeNested],
         # because numpy does the same?
-        List[Any],
+        list[Any],
         # anything with a dtype attribute
-        _SupportsDType,
+        _SupportsDType[np.dtype[Any]],
     ]
     try:
         from cftime import datetime as CFTimeDatetime
@@ -172,6 +161,7 @@ CFCalendar = Literal[
 
 CoarsenBoundaryOptions = Literal["exact", "trim", "pad"]
 SideOptions = Literal["left", "right"]
+InclusiveOptions = Literal["both", "neither", "left", "right"]
 
 ScaleOptions = Literal["linear", "symlog", "log", "logit", None]
 HueStyleOptions = Literal["continuous", "discrete", None]
