@@ -1,6 +1,7 @@
 from __future__ import annotations
 
-from typing import TYPE_CHECKING, Generic, Hashable, Iterable, Literal, Sequence, cast
+from collections.abc import Hashable, Iterable, Sequence
+from typing import TYPE_CHECKING, Generic, Literal, cast
 
 import numpy as np
 from numpy.typing import ArrayLike
@@ -342,7 +343,6 @@ class Weighted(Generic[T_Xarray]):
             skipna: bool,
             method: QUANTILE_METHODS = "linear",
         ) -> np.ndarray:
-
             # This algorithm has been adapted from:
             #   https://aakinshin.net/posts/weighted-quantiles/#reference-implementation
             is_nan = np.isnan(data)
@@ -443,7 +443,6 @@ class Weighted(Generic[T_Xarray]):
         return result
 
     def _implementation(self, func, dim, **kwargs):
-
         raise NotImplementedError("Use `Dataset.weighted` or `DataArray.weighted`")
 
     def sum_of_weights(
@@ -451,7 +450,6 @@ class Weighted(Generic[T_Xarray]):
         dim: Dims = None,
         keep_attrs: bool | None = None,
     ) -> T_Xarray:
-
         return self._implementation(
             self._sum_of_weights, dim=dim, keep_attrs=keep_attrs
         )
@@ -462,7 +460,6 @@ class Weighted(Generic[T_Xarray]):
         skipna: bool | None = None,
         keep_attrs: bool | None = None,
     ) -> T_Xarray:
-
         return self._implementation(
             self._sum_of_squares, dim=dim, skipna=skipna, keep_attrs=keep_attrs
         )
@@ -473,7 +470,6 @@ class Weighted(Generic[T_Xarray]):
         skipna: bool | None = None,
         keep_attrs: bool | None = None,
     ) -> T_Xarray:
-
         return self._implementation(
             self._weighted_sum, dim=dim, skipna=skipna, keep_attrs=keep_attrs
         )
@@ -484,7 +480,6 @@ class Weighted(Generic[T_Xarray]):
         skipna: bool | None = None,
         keep_attrs: bool | None = None,
     ) -> T_Xarray:
-
         return self._implementation(
             self._weighted_mean, dim=dim, skipna=skipna, keep_attrs=keep_attrs
         )
@@ -495,7 +490,6 @@ class Weighted(Generic[T_Xarray]):
         skipna: bool | None = None,
         keep_attrs: bool | None = None,
     ) -> T_Xarray:
-
         return self._implementation(
             self._weighted_var, dim=dim, skipna=skipna, keep_attrs=keep_attrs
         )
@@ -506,7 +500,6 @@ class Weighted(Generic[T_Xarray]):
         skipna: bool | None = None,
         keep_attrs: bool | None = None,
     ) -> T_Xarray:
-
         return self._implementation(
             self._weighted_std, dim=dim, skipna=skipna, keep_attrs=keep_attrs
         )
@@ -519,7 +512,6 @@ class Weighted(Generic[T_Xarray]):
         keep_attrs: bool | None = None,
         skipna: bool = True,
     ) -> T_Xarray:
-
         return self._implementation(
             self._weighted_quantile, q=q, dim=dim, skipna=skipna, keep_attrs=keep_attrs
         )
@@ -534,7 +526,6 @@ class Weighted(Generic[T_Xarray]):
 
 class DataArrayWeighted(Weighted["DataArray"]):
     def _implementation(self, func, dim, **kwargs) -> DataArray:
-
         self._check_dim(dim)
 
         dataset = self.obj._to_temp_dataset()
@@ -544,14 +535,12 @@ class DataArrayWeighted(Weighted["DataArray"]):
 
 class DatasetWeighted(Weighted["Dataset"]):
     def _implementation(self, func, dim, **kwargs) -> Dataset:
-
         self._check_dim(dim)
 
         return self.obj.map(func, dim=dim, **kwargs)
 
 
 def _inject_docstring(cls, cls_name):
-
     cls.sum_of_weights.__doc__ = _SUM_OF_WEIGHTS_DOCSTRING.format(cls=cls_name)
 
     cls.sum.__doc__ = _WEIGHTED_REDUCE_DOCSTRING_TEMPLATE.format(
